@@ -18,7 +18,7 @@ Author: Martin Renters.
 
 3c509 support added by Serge Babkin (babkin@hq.icb.chel.su)
 
-$Id: 3c509.c,v 1.3 2000/07/29 19:22:54 okuji Exp $
+$Id: 3c509.c,v 1.4 2002/01/02 21:56:40 okuji Exp $
 
 ***************************************************************************/
 
@@ -27,11 +27,13 @@ $Id: 3c509.c,v 1.3 2000/07/29 19:22:54 okuji Exp $
 #include "etherboot.h"
 #include "nic.h"
 #include "cards.h"
+#include "timer.h"
 #include "3c509.h"
 
-static unsigned char	eth_vendor, eth_flags, eth_laar;
-static unsigned short	eth_nic_base, eth_asic_base;
-static char		bnc=0, utp=0; /* for 3C509 */
+#define	udelay(n)	waiton_timer2(((n)*TICKS_PER_MS)/1000)
+
+static unsigned short	eth_nic_base;
+static enum { none, bnc, utp } connector = none;	/* for 3C509 */
 
 #ifdef	INCLUDE_3C529
 /*
@@ -55,17 +57,6 @@ static struct el3_mca_adapters_struct el3_mca_adapters[] = {
 };
 #endif
 
-/* a surrogate */
-
-static void DELAY(int val)
-{
-	int c;
-
-	for(c=0; c<val; c+=20) {
-		twiddle();
-	}
-}
-
 /**************************************************************************
 ETH_RESET - Reset adapter
 ***************************************************************************/
@@ -77,15 +68,14 @@ static void t509_reset(struct nic *nic)
 			Reset 3Com 509 card
 	*************************************************************/
 
-	if (eth_vendor != VENDOR_3C509)
-		return;
-
 	/* stop card */
 	outw(RX_DISABLE, BASE + EP_COMMAND);
 	outw(RX_DISCARD_TOP_PACK, BASE + EP_COMMAND);
-	while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
+	while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS)
+		;
 	outw(TX_DISABLE, BASE + EP_COMMAND);
 	outw(STOP_TRANSCEIVER, BASE + EP_COMMAND);
+	udelay(1000);
 	outw(RX_RESET, BASE + EP_COMMAND);
 	outw(TX_RESET, BASE + EP_COMMAND);
 	outw(C_INTR_LATCH, BASE + EP_COMMAND);
@@ -96,7 +86,8 @@ static void t509_reset(struct nic *nic)
 	/*
 	* initialize card
 	*/
-	while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
+	while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS)
+		;
 
 	GO_WINDOW(0);
 
@@ -112,7 +103,7 @@ static void t509_reset(struct nic *nic)
 	GO_WINDOW(2);
 
 	/* Reload the ether_addr. */
-	for (i = 0; i < ETHER_ADDR_SIZE; i++)
+	for (i = 0; i < ETH_ALEN; i++)
 		outb(nic->node_addr[i], BASE + EP_W2_ADDR_0 + i);
 
 	outw(RX_RESET, BASE + EP_COMMAND);
@@ -133,24 +124,24 @@ static void t509_reset(struct nic *nic)
 	outw(SET_RX_FILTER | FIL_INDIVIDUAL | FIL_BRDCST, BASE + EP_COMMAND);
 
 	/* configure BNC */
-	if (bnc) {
+	if (connector == bnc) {
 		outw(START_TRANSCEIVER, BASE + EP_COMMAND);
-		DELAY(10000);
-		}
+		udelay(1000);
+	}
 	/* configure UTP */
-	if (utp) {
+	else if (connector == utp) {
 		GO_WINDOW(4);
 		outw(ENABLE_UTP, BASE + EP_W4_MEDIA_TYPE);
+		sleep(2);	/* Give time for media to negotiate */
 		GO_WINDOW(1);
-		}
+	}
 
-	/* start tranciever and receiver */
+	/* start transceiver and receiver */
 	outw(RX_ENABLE, BASE + EP_COMMAND);
 	outw(TX_ENABLE, BASE + EP_COMMAND);
 
 	/* set early threshold for minimal packet length */
-	outw(SET_RX_EARLY_THRESH | ETH_MIN_PACKET, BASE + EP_COMMAND);
-
+	outw(SET_RX_EARLY_THRESH | ETH_ZLEN, BASE + EP_COMMAND);
 	outw(SET_TX_START_THRESH | 16, BASE + EP_COMMAND);
 }
 
@@ -171,17 +162,14 @@ const char *p)			/* Packet */
 	int pad;
 	int status;
 
-	if(eth_vendor != VENDOR_3C509)
-		return;
-
 #ifdef	EDEBUG
-	printf("{l=%d,t=%x}",s+ETHER_HDR_SIZE,t);
+	printf("{l=%d,t=%hX}",s+ETH_HLEN,t);
 #endif
 
 	/* swap bytes of type */
 	t= htons(t);
 
-	len=s+ETHER_HDR_SIZE; /* actual length of packet */
+	len=s+ETH_HLEN; /* actual length of packet */
 	pad = padmap[len & 3];
 
 	/*
@@ -189,30 +177,28 @@ const char *p)			/* Packet */
 	* but we drop packets that are too large. Perhaps we should truncate
 	* them instead?
 	*/
-	if (len + pad > ETH_MAX_PACKET) {
+	if (len + pad > ETH_FRAME_LEN) {
 		return;
 	}
 
 	/* drop acknowledgements */
-	while(( status=inb(BASE + EP_W1_TX_STATUS) )& TXS_COMPLETE ) {
-		if(status & (TXS_UNDERRUN|TXS_MAX_COLLISION|TXS_STATUS_OVERFLOW)) {
+	while ((status=inb(BASE + EP_W1_TX_STATUS)) & TXS_COMPLETE ) {
+		if (status & (TXS_UNDERRUN|TXS_MAX_COLLISION|TXS_STATUS_OVERFLOW)) {
 			outw(TX_RESET, BASE + EP_COMMAND);
 			outw(TX_ENABLE, BASE + EP_COMMAND);
 		}
-
 		outb(0x0, BASE + EP_W1_TX_STATUS);
 	}
 
-	while (inw(BASE + EP_W1_FREE_TX) < (unsigned short)len + pad + 4) {
-		/* no room in FIFO */
-	}
+	while (inw(BASE + EP_W1_FREE_TX) < (unsigned short)len + pad + 4)
+		; /* no room in FIFO */
 
 	outw(len, BASE + EP_W1_TX_PIO_WR_1);
 	outw(0x0, BASE + EP_W1_TX_PIO_WR_1);	/* Second dword meaningless */
 
 	/* write packet */
-	outsw(BASE + EP_W1_TX_PIO_WR_1, d, ETHER_ADDR_SIZE/2);
-	outsw(BASE + EP_W1_TX_PIO_WR_1, nic->node_addr, ETHER_ADDR_SIZE/2);
+	outsw(BASE + EP_W1_TX_PIO_WR_1, d, ETH_ALEN/2);
+	outsw(BASE + EP_W1_TX_PIO_WR_1, nic->node_addr, ETH_ALEN/2);
 	outw(t, BASE + EP_W1_TX_PIO_WR_1);
 	outsw(BASE + EP_W1_TX_PIO_WR_1, p, s / 2);
 	if (s & 1)
@@ -221,8 +207,9 @@ const char *p)			/* Packet */
 	while (pad--)
 		outb(0, BASE + EP_W1_TX_PIO_WR_1);	/* Padding */
 
-	/* timeout after sending */
-	DELAY(1000);
+	/* wait for Tx complete */
+	while((inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS) != 0)
+		;
 }
 
 /**************************************************************************
@@ -236,14 +223,11 @@ static int t509_poll(struct nic *nic)
 	short status, cst;
 	register short rx_fifo;
 
-	if(eth_vendor!=VENDOR_3C509)
-		return 0;
-
 	cst=inw(BASE + EP_STATUS);
 
 #ifdef	EDEBUG
 	if(cst & 0x1FFF)
-		printf("-%x-",cst);
+		printf("-%hX-",cst);
 #endif
 
 	if( (cst & S_RX_COMPLETE)==0 ) {
@@ -256,7 +240,7 @@ static int t509_poll(struct nic *nic)
 
 	status = inw(BASE + EP_W1_RX_STATUS);
 #ifdef	EDEBUG
-	printf("*%x*",status);
+	printf("*%hX*",status);
 #endif
 
 	if (status & ERR_RX) {
@@ -280,10 +264,9 @@ static int t509_poll(struct nic *nic)
 	while(1) {
 		status = inw(BASE + EP_W1_RX_STATUS);
 #ifdef	EDEBUG
-		printf("*%x*",status);
+		printf("*%hX*",status);
 #endif
 		rx_fifo = status & RX_BYTES_MASK;
-
 		if(rx_fifo>0) {
 			insw(BASE + EP_W1_RX_PIO_RD_1, nic->packet+nic->packetlen, rx_fifo / 2);
 			if(rx_fifo & 1)
@@ -293,32 +276,29 @@ static int t509_poll(struct nic *nic)
 			printf("+%d",rx_fifo);
 #endif
 		}
-
 		if(( status & RX_INCOMPLETE )==0) {
 #ifdef	EDEBUG
 			printf("=%d",nic->packetlen);
 #endif
 			break;
 		}
-
-		DELAY(1000);
+		udelay(1000);	/* if incomplete wait 1 ms */
 	}
-
 	/* acknowledge reception of packet */
 	outw(RX_DISCARD_TOP_PACK, BASE + EP_COMMAND);
-	while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS);
+	while (inw(BASE + EP_STATUS) & S_COMMAND_IN_PROGRESS)
+		;
 #ifdef	EDEBUG
 	type = (nic->packet[12]<<8) | nic->packet[13];
 	if(nic->packet[0]+nic->packet[1]+nic->packet[2]+nic->packet[3]+nic->packet[4]+
-	    nic->packet[5] == 0xFF*ETHER_ADDR_SIZE)
-		printf(",t=0x%x,b]",type);
+	    nic->packet[5] == 0xFF*ETH_ALEN)
+		printf(",t=%hX,b]",type);
 	else
-		printf(",t=0x%x]",type);
+		printf(",t=%hX]",type);
 #endif
-	return 1;
+	return (1);
 }
 
-
 /*************************************************************************
 	3Com 509 - specific routines
 **************************************************************************/
@@ -384,7 +364,8 @@ get_eeprom_data(int id_port, int offset)
 {
 	int i, data = 0;
 	outb(0x80 + offset, id_port);
-	DELAY(10000);
+	/* Do we really need this wait? Won't be noticeable anyway */
+	udelay(10000);
 	for (i = 0; i < 16; i++)
 		data = (data << 1) | (inw(id_port) & 1);
 	return (data);
@@ -415,11 +396,10 @@ struct nic *t509_probe(struct nic *nic, unsigned short *probe_addrs)
 
 	t509_disable(nic);		/* in case board was active */
 					/* note that nic is not used */
-
-	for (failcount=0; failcount<4000; failcount++) {
-		int data, j, io_base, id_port = EP_ID_PORT;
+	for (failcount = 0; failcount < 4000; failcount++) {
+		int data, j, io_base, id_port;
 		unsigned short k;
-		int ep_current_tag = EP_LAST_TAG + 1;
+		int ep_current_tag;
 		short *p;
 #ifdef	INCLUDE_3C529
 		int curboard;
@@ -427,7 +407,6 @@ struct nic *t509_probe(struct nic *nic, unsigned short *probe_addrs)
 
 		id_port = EP_ID_PORT;
 		ep_current_tag = EP_LAST_TAG + 1;
-		eth_vendor = VENDOR_NONE;
 
 	/*********************************************************
 			Search for 3Com 509 card
@@ -476,8 +455,6 @@ struct nic *t509_probe(struct nic *nic, unsigned short *probe_addrs)
 		outb_p(0, MCA_ADAPTER_SETUP_REG);
 
 		if (mcafound) {
-			eth_vendor = VENDOR_3C509;
-
 			eth_nic_base = ((short)((mca_pos4&0xfc)|0x02)) << 8;
 			mca_irq = mca_pos5 & 0x0f;
 			ep_current_tag--;
@@ -487,18 +464,18 @@ struct nic *t509_probe(struct nic *nic, unsigned short *probe_addrs)
 #endif
 	/* Look for the EISA boards, leave them activated */
 	/* search for the first card, ignore all others */
-	for(j = 1; j < 16 && eth_vendor==VENDOR_NONE ; j++) {
+	for(j = 1; j < 16; j++) {
 		io_base = (j * EP_EISA_START) | EP_EISA_W0;
 		if (inw(io_base + EP_W0_MFG_ID) != MFG_ID)
 			continue;
 
-		/* we must found 0x1f if the board is EISA configurated */
+		/* we must have found 0x1f if the board is EISA configurated */
 		if ((inw(io_base + EP_W0_ADDRESS_CFG) & 0x1f) != 0x1f)
 			continue;
 
 		/* Reset and Enable the card */
 		outb(W0_P4_CMD_RESET_ADAPTER, io_base + EP_W0_CONFIG_CTRL);
-		DELAY(10000); /* we must wait at least 10 ms */
+		udelay(1000); /* Must wait 800 s, be conservative */
 		outb(W0_P4_CMD_ENABLE_ADAPTER, io_base + EP_W0_CONFIG_CTRL);
 
 		/*
@@ -506,15 +483,15 @@ struct nic *t509_probe(struct nic *nic, unsigned short *probe_addrs)
 		 * x000 - x00F, where x is the slot number.
 		 */
 		eth_nic_base = j * EP_EISA_START;
-		eth_vendor = VENDOR_3C509;
+		break;
 	}
 	ep_current_tag--;
 
 	/* Look for the ISA boards. Init and leave them actived */
 	/* search for the first card, ignore all others */
 	outb(0xc0, id_port);	/* Global reset */
-	DELAY(10000);
-	for (i = 0; i < EP_MAX_BOARDS && eth_vendor==VENDOR_NONE; i++) {
+	udelay(1000);		/* wait 1 ms */
+	for (i = 0; i < EP_MAX_BOARDS; i++) {
 		outb(0, id_port);
 		outb(0, id_port);
 		send_ID_sequence(id_port);
@@ -531,11 +508,11 @@ struct nic *t509_probe(struct nic *nic, unsigned short *probe_addrs)
 		    (get_eeprom_data(id_port, EEPROM_ADDR_CFG) & 0x1f) * 0x10 + 0x200;
 		outb(ep_current_tag, id_port);	/* tags board */
 		outb(ACTIVATE_ADAPTER_TO_CONFIG, id_port);
-		eth_vendor = VENDOR_3C509;
 		ep_current_tag--;
+		break;
 	}
 
-	if(eth_vendor != VENDOR_3C509)
+	if (i >= EP_MAX_BOARDS)
 		goto no3c509;
 
 	/*
@@ -550,7 +527,7 @@ struct nic *t509_probe(struct nic *nic, unsigned short *probe_addrs)
 	 */
 	if (mcafound) {
 		if (mcafound->id != k) {
-			printf("MCA: PROD_ID in EEPROM does not match MCA card ID! (%x != %x)\n", k, mcafound->id);
+			printf("MCA: PROD_ID in EEPROM does not match MCA card ID! (%hX != %hX)\n", k, mcafound->id);
 			goto no3c509;
 		}
 	} else { /* for ISA/EISA */
@@ -564,15 +541,14 @@ struct nic *t509_probe(struct nic *nic, unsigned short *probe_addrs)
 
 #ifdef	INCLUDE_3C529
 	if (mcafound) {
-		printf("%s board found on MCA at 0x%x IRQ %d -",
+		printf("%s board found on MCA at %#hx IRQ %d -",
 		       mcafound->name, eth_nic_base, mca_irq);
 	} else {
 #endif
-		if(eth_nic_base >= EP_EISA_START) {
-			printf("3C5x9 board on EISA at 0x%x - ",eth_nic_base);
-		} else {
-			printf("3C5x9 board on ISA at 0x%x - ",eth_nic_base);
-		}
+		if(eth_nic_base >= EP_EISA_START)
+			printf("3C5x9 board on EISA at %#hx - ",eth_nic_base);
+		else
+			printf("3C5x9 board on ISA at %#hx - ",eth_nic_base);
 #ifdef	INCLUDE_3C529
 	}
 #endif
@@ -583,61 +559,48 @@ struct nic *t509_probe(struct nic *nic, unsigned short *probe_addrs)
 
 	switch(j) {
 		case 0:
-			if(i & IS_UTP) {
+			if (i & IS_UTP) {
 				printf("10baseT\n");
-				utp=1;
+				connector = utp;
 				}
 			else {
 				printf("10baseT not present\n");
-				eth_vendor=VENDOR_NONE;
 				goto no3c509;
 				}
-
 			break;
 		case 1:
-			if(i & IS_AUI)
+			if (i & IS_AUI)
 				printf("10base5\n");
 			else {
 				printf("10base5 not present\n");
-				eth_vendor=VENDOR_NONE;
 				goto no3c509;
 				}
-
 			break;
 		case 3:
-			if(i & IS_BNC) {
+			if (i & IS_BNC) {
 				printf("10base2\n");
-				bnc=1;
+				connector = bnc;
 				}
 			else {
 				printf("10base2 not present\n");
-				eth_vendor=VENDOR_NONE;
 				goto no3c509;
 				}
-
 			break;
 		default:
 			printf("unknown connector\n");
-			eth_vendor=VENDOR_NONE;
 			goto no3c509;
 		}
 	/*
 	* Read the station address from the eeprom
 	*/
 	p = (unsigned short *) nic->node_addr;
-	for (i = 0; i < 3; i++) {
+	for (i = 0; i < ETH_ALEN / 2; i++) {
 		GO_WINDOW(0);
 		p[i] = htons(get_e(i));
 		GO_WINDOW(2);
 		outw(ntohs(p[i]), BASE + EP_W2_ADDR_0 + (i * 2));
 	}
-
-	printf("Ethernet address: ");
-	for(i=0; i<5; i++) {
-		printf("%b:",nic->node_addr[i]);
-	}
-	printf("%b\n",nic->node_addr[i]);
-
+	printf("Ethernet address: %!\n", nic->node_addr);
 	t509_reset(nic);
 	nic->reset = t509_reset;
 	nic->poll = t509_poll;
@@ -645,7 +608,6 @@ struct nic *t509_probe(struct nic *nic, unsigned short *probe_addrs)
 	nic->disable = t509_disable;
 	return nic;
 no3c509:
-	eth_vendor = VENDOR_NONE;
 	printf("(probe fail)");
 	}
 	return 0;
@@ -656,4 +618,3 @@ no3c509:
  *  c-basic-offset: 8
  * End:
  */
-
